@@ -1,16 +1,30 @@
 # coding=utf-8
 
-from __future__ import unicode_literals, absolute_import, print_function, division
+from __future__ import absolute_import, division, print_function, unicode_literals
 
 import os.path
 
 from sopel.config.types import (
-    StaticSection, ValidatedAttribute, ListAttribute, ChoiceAttribute,
-    FilenameAttribute, NO_DEFAULT
+    BooleanAttribute,
+    ChoiceAttribute,
+    FilenameAttribute,
+    ListAttribute,
+    NO_DEFAULT,
+    SecretAttribute,
+    StaticSection,
+    ValidatedAttribute,
 )
 from sopel.tools import Identifier
 
 
+COMMAND_DEFAULT_PREFIX = r'\.'
+"""Default prefix used for commands."""
+COMMAND_DEFAULT_HELP_PREFIX = '.'
+"""Default help prefix used in commands' usage messages."""
+URL_DEFAULT_SCHEMES = ['http', 'https', 'ftp']
+"""Default URL schemes allowed for URLs."""
+
+
 def _find_certs():
     """Find the TLS root CA store.
 
@@ -65,11 +79,28 @@ class CoreSection(StaticSection):
     """The config section used for configuring the bot itself.
 
     .. important::
+
         All **Required** values must be specified, or Sopel will fail to start.
+
+    .. note::
+
+        You can use the command ``sopel configure`` to generate a config file
+        with the minimal required options.
+
     """
 
     admins = ListAttribute('admins')
-    """The list of people (other than the owner) who can administer the bot."""
+    """The list of people (other than the owner) who can administer the bot.
+
+    Example:
+
+    .. code-block:: ini
+
+        admin =
+            YourFavAdmin
+            TheOtherAdmin
+            YetAnotherRockstarAdmin
+    """
 
     admin_accounts = ListAttribute('admin_accounts')
     """The list of admin accounts other than the owner's.
@@ -77,9 +108,20 @@ class CoreSection(StaticSection):
     Each account is allowed to administer the bot and can perform commands
     that are restricted to admins.
 
+    Example:
+
+    .. code-block:: ini
+
+        admin_accounts =
+            favadmin
+            otheradm
+            yetanotherone
+
     .. important::
+
         This should not be set for networks that do not support IRCv3 account
-        capabilities.
+        capabilities. In that case, use :attr:`admins` instead.
+
     """
 
     alias_nicks = ListAttribute('alias_nicks')
@@ -88,9 +130,18 @@ class CoreSection(StaticSection):
     These aliases are used along with the bot's nick for ``$nick`` and
     ``$nickname`` regex substitutions.
 
-    For example, a bot named "William" (its :attr:`nick`) could have aliases
-    "Bill", "Will", and "Liam". This would then allow both "William: Hi!" and
-    "Bill: Hi!" to work with :func:`~sopel.module.nickname_commands`.
+    For example, a bot named "William" (its :attr:`nick`) could have these
+    aliases:
+
+    .. code-block:: ini
+
+        alias_nicks =
+            Bill
+            Will
+            Liam
+
+    This would then allow both "William: Hi!" and "Bill: Hi!" to work with
+    :func:`~sopel.plugin.nickname_command`.
     """
 
     auth_method = ChoiceAttribute('auth_method', choices=[
@@ -111,7 +162,7 @@ class CoreSection(StaticSection):
         value will override :attr:`server_auth_method`.
     """
 
-    auth_password = ValidatedAttribute('auth_password')
+    auth_password = SecretAttribute('auth_password')
     """The password to use to authenticate with the :attr:`auth_method`.
 
     See :ref:`Authentication`.
@@ -140,41 +191,85 @@ class CoreSection(StaticSection):
     auto_url_schemes = ListAttribute(
         'auto_url_schemes',
         strip=True,
-        default=['http', 'https', 'ftp'])
+        default=URL_DEFAULT_SCHEMES)
     """List of URL schemes that will trigger URL callbacks.
 
     :default: ``['http', 'https', 'ftp']``
 
     Used by the URL callbacks feature to call plugins when links are posted in
-    chat; see the :func:`sopel.module.url` decorator.
+    chat; see the :func:`sopel.plugin.url` decorator.
+
+    The default value allows ``http``, ``https``, and ``ftp``. It is equivalent
+    to this configuration example:
+
+    .. code-block:: ini
+
+        auto_url_schemes =
+            http
+            https
+            ftp
 
-    The default value allows ``http``, ``https``, and ``ftp``.
     """
 
     bind_host = ValidatedAttribute('bind_host')
     """Bind the connection to a specific IP.
 
     :default: ``0.0.0.0`` (all interfaces)
+
+    This is equivalent to the default value:
+
+    .. code-block:: ini
+
+        bind_host = 0.0.0.0
+
     """
 
     ca_certs = FilenameAttribute('ca_certs', default=_find_certs())
     """The path to the CA certs ``.pem`` file.
 
-    If not specified, Sopel will try to find the certificate trust store itself.
+    Example:
+
+    .. code-block:: ini
+
+        ca_certs = /etc/ssl/certs/ca-certificates.crt
+
+    If not specified, Sopel will try to find the certificate trust store
+    itself from a set of known locations.
+
+    If the given value is not an absolute path, it will be interpreted relative
+    to the directory containing the config file with which Sopel was started.
     """
 
     channels = ListAttribute('channels')
     """List of channels for the bot to join when it connects.
 
     If a channel key needs to be provided, separate it from the channel name
-    with a space, e.g. ``"#channel password"``.
+    with a space:
+
+    .. code-block:: ini
+
+        channels =
+            "#channel"
+            "#logs"
+            &rare_prefix_channel
+            "#private password"
+
+    .. important::
+
+        If you edit the config file manually, make sure to wrap each line
+        starting with a ``#`` in double quotes, as shown in the example above.
+        An unquoted ``#`` denotes a comment, which will be ignored by Sopel's
+        configuration parser.
+
     """
 
     commands_on_connect = ListAttribute('commands_on_connect')
     """A list of commands to send upon successful connection to the IRC server.
 
-    Each line is a message that will be sent to the server once connected.
-    Example::
+    Each line is a message that will be sent to the server once connected,
+    in the order they are defined:
+
+    .. code-block:: ini
 
         commands_on_connect =
             PRIVMSG Q@CServe.quakenet.org :AUTH my_username MyPassword,@#$%!
@@ -218,7 +313,7 @@ class CoreSection(StaticSection):
     Ignored when using SQLite.
     """
 
-    db_pass = ValidatedAttribute('db_pass')
+    db_pass = SecretAttribute('db_pass')
     """The password for Sopel's database.
 
     Ignored when using SQLite.
@@ -260,6 +355,12 @@ class CoreSection(StaticSection):
     mssql
       ``pip install pymssql``
 
+    This is equivalent to the default value:
+
+    .. code-block:: ini
+
+        db_type = sqlite
+
     .. seealso::
 
         Refer to :ref:`SQLAlchemy's documentation <dialect_toplevel>` for more
@@ -286,6 +387,12 @@ class CoreSection(StaticSection):
 
     Used when plugins format times with :func:`sopel.tools.time.format_time`.
 
+    This is equivalent to the default value:
+
+    .. code-block:: ini
+
+        default_time_format = %Y-%m-%d - %T%Z
+
     .. seealso::
 
         Time format reference is available in the documentation for Python's
@@ -298,37 +405,91 @@ class CoreSection(StaticSection):
 
     :default: ``UTC``
 
+    .. highlight:: ini
+
     Used when plugins format times with :func:`sopel.tools.time.format_time`.
+
+    For example, to make Sopel fall back on British time::
+
+        default_timezone = Europe/London
+
+    And this is equivalent to the default value::
+
+        default_timezone = UTC
+
     """
 
     enable = ListAttribute('enable')
     """A list of the only plugins you want to enable.
 
-    If set, Sopel will *only* load the plugins named here. All other available
-    plugins will be ignored.
+    .. highlight:: ini
 
-    To load *all* available plugins, clear this setting.
+    If set, Sopel will *only* load the plugins named here. All other available
+    plugins will be ignored::
+
+        enable =
+            url
+            xkcd
+            help
+
+    In that case, only the ``url``, ``xkcd``, and ``help`` plugins will be
+    enabled and loaded by Sopel.
+
+    To load *all* available plugins, clear this setting by removing it, or
+    by making it empty::
+
+        enable =
 
     To disable only a few plugins, see :attr:`exclude`.
 
-    See :ref:`Plugins` for an overview of all plugin-related settings.
+    .. seealso::
+
+        The :ref:`Plugins` chapter for an overview of all plugin-related
+        settings.
+
     """
 
     exclude = ListAttribute('exclude')
     """A list of plugins which should not be loaded.
 
-    If set, Sopel will load all available plugins *except* those named here.
+    .. highlight:: ini
+
+    If set, Sopel will load all available plugins *except* those named here::
+
+        exclude =
+            url
+            calc
+            meetbot
+
+    In that case, ``url``, ``calc``, and ``meetbot`` will be excluded, and they
+    won't be loaded by Sopel.
 
     A plugin named both here and in :attr:`enable` **will not** be loaded;
     :attr:`exclude` takes priority.
 
-    See :ref:`Plugins` for an overview of all plugin-related settings.
+    .. seealso::
+
+        The :ref:`Plugins` chapter for an overview of all plugin-related
+        settings.
+
     """
 
     extra = ListAttribute('extra')
     """A list of other directories in which to search for plugin files.
 
-    See :ref:`Plugins` for an overview of all plugin-related settings.
+    Example:
+
+    .. code-block:: ini
+
+        extra =
+            /home/myuser/custom-sopel-plugins/
+            /usr/local/lib/ad-hoc-plugins/
+
+    .. seealso::
+
+        The :ref:`Plugins` chapter for an overview of all plugin-related
+        settings.
+
     """
 
     flood_burst_lines = ValidatedAttribute('flood_burst_lines', int, default=4)
@@ -336,7 +497,16 @@ class CoreSection(StaticSection):
 
     :default: ``4``
 
-    See :ref:`Flood Prevention` to learn what each flood-related setting does.
+    This is equivalent to the default value:
+
+    .. code-block:: ini
+
+        flood_burst_lines = 4
+
+    .. seealso::
+
+        The :ref:`Flood Prevention` chapter to learn what each flood-related
+        setting does.
 
     .. versionadded:: 7.0
     """
@@ -346,26 +516,124 @@ class CoreSection(StaticSection):
 
     :default: ``0.7``
 
-    See :ref:`Flood Prevention` to learn what each flood-related setting does.
+    This is equivalent to the default value:
+
+    .. code-block:: ini
+
+        flood_empty_wait = 0.7
+
+    .. seealso::
+
+        The :ref:`Flood Prevention` chapter to learn what each flood-related
+        setting does.
 
     .. versionadded:: 7.0
     """
 
+    flood_max_wait = ValidatedAttribute('flood_max_wait', float, default=2)
+    """How much time to wait at most when flood protection kicks in.
+
+    :default: ``2``
+
+    This is equivalent to the default value:
+
+    .. code-block:: ini
+
+        flood_max_wait = 2
+
+    .. seealso::
+
+        The :ref:`Flood Prevention` chapter to learn what each flood-related
+        setting does.
+
+    .. versionadded:: 7.1
+    """
+
+    flood_penalty_ratio = ValidatedAttribute('flood_penalty_ratio',
+                                             float,
+                                             default=1.4)
+    """Ratio of the message length used to compute the added wait penalty.
+
+    :default: ``1.4``
+
+    Messages longer than :attr:`flood_text_length` will get an added
+    wait penalty (in seconds) that will be computed like this::
+
+        overflow = max(0, (len(text) - flood_text_length))
+        rate = flood_text_length * flood_penalty_ratio
+        penalty = overflow / rate
+
+    .. note::
+
+        If the penalty ratio is 0, this penalty will be disabled.
+
+    This is equivalent to the default value:
+
+    .. code-block:: ini
+
+        flood_penalty_ratio = 1.4
+
+    .. seealso::
+
+        The :ref:`Flood Prevention` chapter to learn what each flood-related
+        setting does.
+
+    .. versionadded:: 7.1
+    """
+
     flood_refill_rate = ValidatedAttribute('flood_refill_rate', int, default=1)
     """How quickly burst mode recovers, in messages per second.
 
     :default: ``1``
 
-    See :ref:`Flood Prevention` to learn what each flood-related setting does.
+    This is equivalent to the default value:
+
+    .. code-block:: ini
+
+        flood_refill_rate = 1
+
+    .. seealso::
+
+        The :ref:`Flood Prevention` chapter to learn what each flood-related
+        setting does.
 
     .. versionadded:: 7.0
     """
 
-    help_prefix = ValidatedAttribute('help_prefix', default='.')
+    flood_text_length = ValidatedAttribute('flood_text_length', int, default=50)
+    """Length of text at which an extra wait penalty is added.
+
+    :default: ``50``
+
+    Messages longer than this (in bytes) get an added wait penalty if the
+    flood protection limit is reached.
+
+    This is equivalent to the default value:
+
+    .. code-block:: ini
+
+        flood_text_length = 50
+
+    .. seealso::
+
+        The :ref:`Flood Prevention` chapter to learn what each flood-related
+        setting does.
+
+    .. versionadded:: 7.1
+    """
+
+    help_prefix = ValidatedAttribute('help_prefix',
+                                     default=COMMAND_DEFAULT_HELP_PREFIX)
     """The prefix to use in help output.
 
     :default: ``.``
 
+    This is equivalent to the default value:
+
+    .. code-block:: ini
+
+        help_prefix = .
+
     If :attr:`prefix` is changed from the default, this setting **must** be
     updated to reflect the prefix your bot will actually respond to, or the
     built-in ``help`` functionality will provide incorrect example usage.
@@ -380,39 +648,82 @@ class CoreSection(StaticSection):
         """
         return self._parent.homedir
 
-    host = ValidatedAttribute('host', default='chat.freenode.net')
+    host = ValidatedAttribute('host', default='irc.libera.chat')
     """The IRC server to connect to.
 
-    :default: ``chat.freenode.net``
+    :default: ``irc.libera.chat``
+
+    **Required**:
+
+    .. code-block:: ini
+
+        host = irc.libera.chat
 
-    **Required.**
     """
 
     host_blocks = ListAttribute('host_blocks')
     """A list of hostnames which Sopel should ignore.
 
-    Messages from any user whose connection hostname matches one of these values
-    will be ignored. :ref:`Regular expression syntax <re-syntax>` is supported.
+    Messages from any user whose connection hostname matches one of these
+    values will be ignored. :ref:`Regular expression syntax <re-syntax>`
+    is supported, so remember to escape special characters:
+
+    .. code-block:: ini
+
+        host_blocks =
+            (.+\\.)*domain\\.com
+
+    .. seealso::
+
+        The :attr:`nick_blocks` list can be used to block users by their nick.
+
+    .. note::
+
+        We are working on a better block system; see `issue #1355`__ for more
+        information and update.
+
+    .. __: https://github.com/sopel-irc/sopel/issues/1355
 
-    Also see the :attr:`nick_blocks` list.
     """
 
-    log_raw = ValidatedAttribute('log_raw', bool, default=False)
+    log_raw = BooleanAttribute('log_raw', default=False)
     """Whether a log of raw lines as sent and received should be kept.
 
-    See :ref:`Raw Logs`.
+    :default: ``no``
+
+    To enable this logging:
+
+    .. code-block:: ini
+
+        log_raw = yes
+
+    .. seealso::
+
+        The :ref:`Raw Logs` chapter.
+
     """
 
     logdir = FilenameAttribute('logdir', directory=True, default='logs')
     """Directory in which to place logs.
 
-    See :ref:`Logging`.
+    :default: ``logs``
+
+    If the given value is not an absolute path, it will be interpreted relative
+    to the directory containing the config file with which Sopel was started.
+
+    .. seealso::
+
+        The :ref:`Logging` chapter.
+
     """
 
     logging_channel = ValidatedAttribute('logging_channel', Identifier)
     """The channel to send logging messages to.
 
-    See :ref:`Log to a Channel`.
+    .. seealso::
+
+        The :ref:`Log to a Channel` chapter.
+
     """
 
     logging_channel_datefmt = ValidatedAttribute('logging_channel_datefmt')
@@ -420,14 +731,14 @@ class CoreSection(StaticSection):
 
     If not specified, this falls back to using :attr:`logging_datefmt`.
 
-    See :ref:`Log to a Channel`.
-
-    .. versionadded:: 7.0
     .. seealso::
 
         Time format reference is available in the documentation for Python's
         :func:`time.strftime` function.
 
+        For more information about logging, see :ref:`Log to a Channel`.
+
+    .. versionadded:: 7.0
     """
 
     logging_channel_format = ValidatedAttribute('logging_channel_format')
@@ -435,7 +746,9 @@ class CoreSection(StaticSection):
 
     If not specified, this falls back to using :attr:`logging_format`.
 
-    See :ref:`Log to a Channel`.
+    .. seealso::
+
+        The :ref:`Log to a Channel` chapter.
 
     .. versionadded:: 7.0
     """
@@ -448,7 +761,9 @@ class CoreSection(StaticSection):
 
     If not specified, this falls back to using :attr:`logging_level`.
 
-    See :ref:`Log to a Channel`.
+    .. seealso::
+
+        The :ref:`Log to a Channel` chapter.
 
     .. versionadded:: 7.0
     """
@@ -459,12 +774,12 @@ class CoreSection(StaticSection):
     If not set, the ``datefmt`` argument is not provided, and :mod:`logging`
     will use the Python default.
 
-    .. versionadded:: 7.0
     .. seealso::
 
         Time format reference is available in the documentation for Python's
         :func:`time.strftime` function.
 
+    .. versionadded:: 7.0
     """
 
     logging_format = ValidatedAttribute(
@@ -480,9 +795,17 @@ class CoreSection(StaticSection):
 
         [2019-10-21 12:47:44,272] sopel.irc            INFO     - Connected.
 
-    .. versionadded:: 7.0
+    This is equivalent to the default value:
+
+    .. code-block:: ini
+
+       logging_format = [%(asctime)s] %(name)-20s %(levelname)-8s - %(message)s
+
     .. seealso::
+
         Python's logging format documentation: :ref:`logrecord-attributes`
+
+    .. versionadded:: 7.0
     """
 
     logging_level = ChoiceAttribute('logging_level',
@@ -500,6 +823,13 @@ class CoreSection(StaticSection):
     * ``WARNING``
     * ``INFO``
     * ``DEBUG``
+
+    For example to log only at WARNING level and above:
+
+    .. code-block:: ini
+
+        logging_level = WARNING
+
     """
 
     modes = ValidatedAttribute('modes', default='B')
@@ -522,7 +852,12 @@ class CoreSection(StaticSection):
 
     :default: ``Sopel``
 
-    **Required.**
+    **Required**:
+
+    .. code-block:: ini
+
+        nick = Sopel
+
     """
 
     nick_auth_method = ChoiceAttribute('nick_auth_method', choices=[
@@ -531,15 +866,19 @@ class CoreSection(StaticSection):
 
     Can be one of ``nickserv``, ``authserv``, ``Q``, or ``userserv``.
 
-    See :ref:`Authentication` for more details.
+    .. seealso::
+
+        The :ref:`Authentication` chapter for more details.
 
     .. versionadded:: 7.0
     """
 
-    nick_auth_password = ValidatedAttribute('nick_auth_password')
+    nick_auth_password = SecretAttribute('nick_auth_password')
     """The password to use to authenticate the bot's nick.
 
-    See :ref:`Authentication` for more details.
+    .. seealso::
+
+        The :ref:`Authentication` chapter for more details.
 
     .. versionadded:: 7.0
     """
@@ -550,8 +889,11 @@ class CoreSection(StaticSection):
     :default: ``NickServ`` for ``nickserv`` authentication; ``UserServ`` for
               ``userserv`` authentication
 
-    May not apply, depending on the chosen :attr:`nick_auth_method`. See
-    :ref:`Authentication` for more details.
+    May not apply, depending on the chosen :attr:`nick_auth_method`.
+
+    .. seealso::
+
+        The :ref:`Authentication` chapter for more details.
 
     .. versionadded:: 7.0
     """
@@ -561,8 +903,11 @@ class CoreSection(StaticSection):
 
     :default: the value of :attr:`nick`
 
-    May not apply, depending on the chosen :attr:`nick_auth_method`. See
-    :ref:`Authentication` for more details.
+    May not apply, depending on the chosen :attr:`nick_auth_method`.
+
+    .. seealso::
+
+        The :ref:`Authentication` chapter for more details.
 
     .. versionadded:: 7.0
     """
@@ -571,19 +916,36 @@ class CoreSection(StaticSection):
     """A list of nicks which Sopel should ignore.
 
     Messages from any user whose nickname matches one of these values will be
-    ignored. :ref:`Regular expression syntax <re-syntax>` is supported.
+    ignored. :ref:`Regular expression syntax <re-syntax>` is supported, so
+    remember to escape special characters:
+
+    .. code-block:: ini
+
+        nick_blocks =
+            ExactNick
+            _*RegexMatch_*
+
+    .. seealso::
+
+        The :attr:`host_blocks` list can be used to block users by their host.
+
+    .. note::
+
+        We are working on a better block system; see `issue #1355`__ for more
+        information and update.
+
+    .. __: https://github.com/sopel-irc/sopel/issues/1355
 
-    Also see the :attr:`host_blocks` list.
     """
 
-    not_configured = ValidatedAttribute('not_configured', bool, default=False)
+    not_configured = BooleanAttribute('not_configured', default=False)
     """For package maintainers. Not used in normal configurations.
 
     :default: ``False``
 
     This allows software packages to install a default config file, with this
-    option set to ``True``, so that the bot will not run until it has been
-    properly configured.
+    option set to ``True``, so that commands to start, stop, or restart the bot
+    won't work until the bot has been properly configured.
     """
 
     owner = ValidatedAttribute('owner', default=NO_DEFAULT)
@@ -616,21 +978,65 @@ class CoreSection(StaticSection):
 
     :default: ``6667`` normally; ``6697`` if :attr:`use_ssl` is ``True``
 
-    **Required.**
+    .. highlight:: ini
+
+    **Required**::
+
+        port = 6667
+
+    And usually when SSL is enabled::
+
+        port = 6697
+        use_ssl = yes
+
     """
 
-    prefix = ValidatedAttribute('prefix', default='\\.')
-    """The prefix to add to the beginning of commands.
+    prefix = ValidatedAttribute('prefix', default=COMMAND_DEFAULT_PREFIX)
+    """The prefix to add to the beginning of commands as a regular expression.
 
     :default: ``\\.``
 
-    **Required.**
+    .. highlight:: ini
+
+    **Required**::
+
+        prefix = \\.
+
+    With the default value, users will invoke commands like this:
+
+    .. code-block:: irc
+
+        <nick> .help
+
+    Since it's a regular expression, you can use multiple prefixes::
+
+        prefix = \\.|\\?
+
+    .. important::
+
+        As the prefix is a regular expression, don't forget to escape it when
+        necessary. It is not recommended to use capturing groups, as it
+        **will** create problems with argument parsing for commands.
+
+    .. note::
+
+        Remember to change the :attr:`help_prefix` value accordingly::
+
+            prefix = \\?
+            help_prefix = ?
+
+        In that example, users will invoke commands like this:
+
+        .. code-block:: irc
+
+            <nick> ?help xkcd
+            <Sopel> ?xkcd - Finds an xkcd comic strip
+            <Sopel> Takes one of 3 inputs:
+            [...]
 
-    It is a regular expression (so the default, ``\\.``, means commands start
-    with a period), though using capturing groups will create problems.
     """
 
-    reply_errors = ValidatedAttribute('reply_errors', bool, default=True)
+    reply_errors = BooleanAttribute('reply_errors', default=True)
     """Whether to reply to the sender of a message that triggered an error.
 
     :default: ``True``
@@ -638,11 +1044,12 @@ class CoreSection(StaticSection):
     If ``True``, Sopel will send information about the triggered exception to
     the sender of the message that caused the error.
 
-    If ``False``, Sopel will only log the error and will appear to fail silently
-    from the triggering IRC user's perspective.
+    If ``False``, Sopel will only log the error and will appear to fail
+    silently from the triggering IRC user's perspective.
     """
 
-    server_auth_method = ChoiceAttribute('server_auth_method', choices=['sasl', 'server'])
+    server_auth_method = ChoiceAttribute('server_auth_method',
+                                         choices=['sasl', 'server'])
     """The server authentication method.
 
     Can be ``sasl`` or ``server``.
@@ -650,7 +1057,7 @@ class CoreSection(StaticSection):
     .. versionadded:: 7.0
     """
 
-    server_auth_password = ValidatedAttribute('server_auth_password')
+    server_auth_password = SecretAttribute('server_auth_password')
     """The password to use to authenticate with the server.
 
     .. versionadded:: 7.0
@@ -681,6 +1088,12 @@ class CoreSection(StaticSection):
 
     If not set, or set to 0, Sopel won't slow down the initial join.
 
+    In this example, Sopel will try to join 4 channels at a time:
+
+    .. code-block:: ini
+
+        throttle_join = 4
+
     .. seealso::
 
         :attr:`throttle_wait` controls Sopel's waiting time between joining
@@ -693,8 +1106,14 @@ class CoreSection(StaticSection):
 
     :default: ``1``
 
-    For example, with ``throttle_join = 2`` and ``throttle_wait = 5`` it will
-    wait 5s every 2 channels it joins.
+    In this example:
+
+    .. code-block:: ini
+
+        throttle_wait = 5
+        throttle_join = 2
+
+    Sopel will join 2 channels every 5s.
 
     If :attr:`throttle_join` is ``0``, this setting has no effect.
 
@@ -705,15 +1124,68 @@ class CoreSection(StaticSection):
     """
 
     timeout = ValidatedAttribute('timeout', int, default=120)
-    """The number of seconds acceptable between pings before timing out.
+    """The number of seconds acceptable since the last message before timing out.
 
     :default: ``120``
+
+    You can change the timeout like this:
+
+    .. code-block:: ini
+
+        # increase to 200 seconds
+        timeout = 200
+
     """
 
-    use_ssl = ValidatedAttribute('use_ssl', bool, default=False)
+    timeout_ping_interval = ValidatedAttribute('timeout_ping_interval',
+                                               int,
+                                               default=0)
+    """The number of seconds before sending a PING command to the server.
+
+    :default: (auto)
+
+    The default value is made to send at least 2 PINGs before the bot thinks
+    there is a timeout, given that :attr:`timeout` is 120s by default:
+
+    * t+54s: first PING
+    * t+108s: second PING
+    * t+120s: if no response, then a timeout is detected
+
+    This makes the timeout detection more lenient and forgiving, by allowing a
+    12s window for the server to send something that will prevent a timeout.
+
+    You can change the PING interval like this:
+
+    .. code-block:: ini
+
+        # timeout up to 250s
+        timeout = 250
+        # PING every 80s (about 3 times every 240s + 10s window)
+        timeout_ping_interval = 80
+
+    .. note::
+
+        Internally, the default value is 0 and the value used will be
+        automatically calculated as 45% of the :attr:`timeout`::
+
+            ping_interval = timeout * 0.45
+
+        So for a timeout of 120s it's a PING every 54s. For a timeout of 250s
+        it's a PING every 112.5s.
+
+    """
+
+    use_ssl = BooleanAttribute('use_ssl', default=False)
     """Whether to use a SSL/TLS encrypted connection.
 
     :default: ``False``
+
+    Example with SSL on:
+
+    .. code-block:: ini
+
+        use_ssl = yes
+
     """
 
     user = ValidatedAttribute('user', default='sopel')
@@ -721,11 +1193,24 @@ class CoreSection(StaticSection):
 
     :default: ``sopel``
 
-    **Required.**
+    **Required**:
+
+    .. code-block:: ini
+
+        user = sopel
+
     """
 
-    verify_ssl = ValidatedAttribute('verify_ssl', bool, default=True)
+    verify_ssl = BooleanAttribute('verify_ssl', default=True)
     """Whether to require a trusted certificate for encrypted connections.
 
     :default: ``True``
+
+    Example with SSL on:
+
+    .. code-block:: ini
+
+        use_ssl = yes
+        verify_ssl = yes
+
     """
